#!/usr/bin/perl -w

use strict;
use Getopt::Long;
use MIME::Parser;
use Fcntl;
use DB_File;

use Bayespam::Process;

#
# bayes_spam_check.pl
#
# This file is part of Bayespam. (v0.9.2)
#
# Usage:
#	bayes_spam_check.pl [corpus rating file] [spam forward address]
#
# Given a corpus rating file (generated by bayes_process_email.pl),
#	a spam forward address, and an email (on STDIN), this program will
#	either accept the email, or it will emit the proper exit code to
#	inform qmail to bounce or drop this message.
#
# Copyright (C) 2002 Gary Arnold
#
# Bayespam is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Bayespam is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bayespam; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

my %token_spam_rating;
my @email_tokens;
my %email_token_spam_rating;
my %interesting_tokens;
my @most_interesting;
my $token_count;
my $case_sensitive;
my $rating_file = '';
my $help = 0;

Getopt::Long::Configure ("bundling");
my $goodOptions = GetOptions ( "rating|r=s" => \$rating_file,
							   "help|h|?" => $help );

# Nobody to complain to if we've got the wrong number of arguments, so just pass thru...safer that way.
if ( ( $rating_file eq '' ) || $help ) { usage(); exit 0; }

# Open the database
tie( %token_spam_rating, "DB_File", $rating_file, O_RDONLY, 0666, $DB_BTREE) ||
	die "Could not open database.";

$case_sensitive = $token_spam_rating{ " case " } || 0;

my $parser = new MIME::Parser;
$parser->output_to_core(1);
$parser->extract_uuencode(1);

my $data = $parser->parse(\*STDIN);
my $message = Bayespam::Process::ProcessMimeMessage( $data );

# Rip out HTML comments
$message =~ s/\<\!\-.*\-\>//gs;
$message = lc $message if not $case_sensitive;

foreach (split( /[^\w'$-]+/, $message ) )
{
	# Ignore all-numeric tokens
	if ( !( $_ =~ /^[0-9\-]+$/g ) ) { if (length($_) > 0){push @email_tokens, $_;} }
}

# Rate each email token according to how far from 0.5 it's "spamminess" is
#	ie: how "interesting" is this token?
foreach ( @email_tokens )
{
	if ( length > 0 )
	{
		my $rating = ( split( / /, ($token_spam_rating{ $_ } || "0 0 0.2") ) )[2];
		$email_token_spam_rating{ $_ } = $rating;
		$interesting_tokens{ $_ } = abs( 0.5 - $rating );
	}
}

# Get the indices of the 15 most interesting tokens
@most_interesting = ( sort { $interesting_tokens{$b} <=> $interesting_tokens{$a} } keys %interesting_tokens )[0..14];

my $prod = 1.0;
my $one_minus_prod = 1.0;

# Do the ol' Bayes magic probability on the email
foreach ( @most_interesting )
{
	if ( defined $_ )
	{
		if ( defined $email_token_spam_rating{$_} )
		{
			$prod *= $email_token_spam_rating{$_};
			$one_minus_prod *= ( 1.0 - $email_token_spam_rating{$_} );
		}
	}
}

untie( %token_spam_rating );

my $probability_of_spam;
# Aaaiiieeeee!  Watch out for tiny numbers, which cause perl to go thru the roof
if ( ( $prod + $one_minus_prod ) > 0.0000001 )
{
	$probability_of_spam = $prod / ( $prod + $one_minus_prod );
}
else
{
	$probability_of_spam = 1.0;
}

# Is it spam?
if ( $probability_of_spam > 0.9 )
{
	# Spam, so exit with "1"
	exit 1;
}

# It's OK, so exit nicely
exit 0;

# Print program usage
sub usage
{
    print STDERR << "EOF";

Usage: $0 [-h] -r file
Given a corpus rating file and an email on STDIN, return 1 if the mail is spam,
  return 0 if not.

 -h, --help          : this (help) message
 -r, --rating        : corpus rating file to use

example: $0 --rating bayes_rating.dat

EOF
}
